
#include <Nextion.h>
#include "Arduino.h"
#include <OneWire.h>
#include <DallasTemperature.h>
#include <TinyGPSPlus.h>
#include <SoftwareSerial.h>
#include <BluetoothSetup.h>
#include "UltrasonicProj.h"
#include "TMP75.h"
#include "BluetoothA2DPSink.h"
#include "esp_avrc_api.h"
#include <freertos/FreeRTOS.h>
#include <freertos/task.h>

//Inside temp sensor
TMP75 mySensor;
//LEDS
const int ledPinR = 21;
const int ledPinL = 12;

//RTOS
TaskHandle_t taskHandle1 = NULL;
TaskHandle_t taskHandle2 = NULL;
TaskHandle_t taskHandle3 = NULL;
TaskHandle_t taskHandle4 = NULL;
TaskHandle_t taskHandle5 = NULL;
TaskHandle_t taskHandle6 = NULL;

//GPS
static const int RXPin = 33, TXPin = 32;
static const uint32_t GPSBaud = 9600;
TinyGPSPlus gps;
SoftwareSerial ss(RXPin, TXPin);
int hour;
int minute;
double speed;

//Ultrasonic
int16_t LF;
int16_t RF;
int16_t LR;
int16_t RR;
int16_t LBS;
int16_t RBS;
float ultraTemp;
//Nextion
uint32_t next, myInt = 0;
//Page 1
NexPage p0 = NexPage(0,0,"Main");
NexNumber p0_Ctemp = NexNumber(0,2,"Ctemp");//Coolant temp
NexNumber p0_Otemp = NexNumber(0,10,"Otemp");//Outside Temp
NexNumber p0_Itemp = NexNumber(0,23,"Itemp");//Outside Temp
NexNumber p0_Speed = NexNumber(0,6,"Speed");//Speed
NexNumber p0_Hour = NexNumber(0,12,"Hour");//Hour
NexNumber p0_Minute = NexNumber(0,13,"Minute");//Minute
//Music page 1 buttons
NexButton p0_Prev = NexButton(0,17,"Prev");//LED button
NexButton p0_Next = NexButton(0,18,"Next");//LED button
NexButton p0_Play = NexButton(0,19,"Play");//LED button
NexButton p0_Pause = NexButton(0,20,"Pause");//LED button
//Page 2
NexPage p1 = NexPage(1,0,"Parking");
NexNumber p1_LF = NexNumber(1,5,"LF");//Front left sensor
NexNumber p1_RF = NexNumber(1,6,"RF");//Front right sensor
NexNumber p1_LR = NexNumber(1,7,"LR");//rear left sensor
NexNumber p1_RR = NexNumber(1,8,"RR");//rear right sensor
NexNumber p1_LBS = NexNumber(1,9,"LBS");//driver blindspot
NexNumber p1_RBS = NexNumber(1,10,"RBS");//pass blindspot

//Coolant temp sensor
#define ONE_WIRE_BUS 13
OneWire oneWire(ONE_WIRE_BUS);
DallasTemperature sensors(&oneWire);
int count = 0;
double Ctemp = 0;
int Otemp;
int Itemp;


NexTouch *nex_listen_list[] = {
  &p0_Prev, &p0_Next, &p0_Play, &p0_Pause,NULL
};

//All buttons
//Page 1 buttons
//Music Page buttons
void p0_Next_Press(void *ptr) {
  BluetoothSetup::next();
}
void p0_Prev_Press(void *ptr) {
  BluetoothSetup::previous();
}
void p0_Play_Press(void *ptr) {
  BluetoothSetup::play();
}
void p0_Pause_Press(void *ptr) {
  BluetoothSetup::pause();
}
//GPS speed and time
void displayInfo()
{
  Serial.print(F("Location: ")); 
  if (gps.location.isValid())
  {
    speed = gps.speed.mph();
  }
  else
  {
    speed = 48;
  }

  if (gps.time.isValid())
  {
    //Set hours for nextion
    hour = gps.time.hour();
    hour = hour-4;
    //Set minutes for nextion
    minute = gps.time.minute();
  }
  else
  {
    Serial.print(F("INVALID"));
    hour = 3;
    minute = 45;
  }
}
//Update Nextion Screen with all data values
void do_every_so_often1(void * Parameter) {
  while(1){
    
    //Ultrasonics
    p1_LF.setValue(LF);
    p1_RF.setValue(RF);
    p1_LR.setValue(LR); 
    p1_RR.setValue(RR);
    p1_LBS.setValue(LBS);
    p1_RBS.setValue(RBS);
    p0_Otemp.setValue(ultraTemp);//Outside temp
    vTaskDelay(1000 / portTICK_PERIOD_MS);
  }
}
//Rest of parameters doesn't happen as often
void do_every_so_often2(void * Parameter) {
  while(1){
    //All other Data
    p0_Speed.setValue(speed);//Speed
    //All other Data
    p0_Ctemp.setValue(Ctemp);//Coolant Temp
    p0_Itemp.setValue(Itemp);//Inside Temp
    p0_Hour.setValue(hour);//Hour
    p0_Minute.setValue(minute);//Minute
    vTaskDelay(5000 / portTICK_PERIOD_MS);
  }
}
//Collecting Ultrasonic Data
void getUltrasonics(void * Parameter){
  while(1){
    LF = UltrasonicProj::receiveUltraOne();//1
    RF = UltrasonicProj::receiveUltraTwo();//2
    LR = UltrasonicProj::receiveUltraThree();//3
    RR = UltrasonicProj::receiveUltraFour();//4
    LBS = UltrasonicProj::receiveUltraFive();//5
    RBS = UltrasonicProj::receiveUltraSix();//6
    ultraTemp = UltrasonicProj::receiveTemperature();//4
    if (50>LBS && LBS>20){
      digitalWrite(ledPinL, HIGH);
    }
    else{
      digitalWrite(ledPinL, LOW);
    }
    if (50>RBS && RBS>20){
      digitalWrite(ledPinR, HIGH);
    }
    else{
      digitalWrite(ledPinR, LOW);
    }
  vTaskDelay(500 / portTICK_PERIOD_MS);
  }
}
//Coolant sensor data
void coolantTemp(void * Parameter)
{
  while(1){
  //Engine Sensor Data
  sensors.requestTemperatures();//Coolant temp
  sensors.getTempFByIndex(0);
  Serial.println(Ctemp);
  vTaskDelay(5000 / portTICK_PERIOD_MS);
  }
}
//Onboard inside temp sensor
void InsideTemp(void * Parameter)
{
  while(1){
  //Get Inside temp
  Itemp = mySensor.readC16();
  Itemp = Itemp * 1.8 + 32;
  vTaskDelay(5000 / portTICK_PERIOD_MS);
  }
}
//Udates Nextion
void NextionList(void * Parameter)
{
  while(1){
  //Updates Nextion screen with values
  nexLoop(nex_listen_list);
  vTaskDelay(100/ portTICK_PERIOD_MS);
  }
}
void GPSAvailable(void * Parameter){
  while(1){
  //Updates Nextion screen with values
  while (ss.available() > 0){
    if (gps.encode(ss.read()))
      displayInfo();
  
  //Updates Nextion screen with values
  }
  vTaskDelay(500 / portTICK_PERIOD_MS);
  }
}

void setup() {
  //Setup Nextion screen
  nexInit();
  //Bluetooth setup
  BluetoothSetup::initializeBluetooth();
  // start serial ports
  Serial.begin(115200);
  Serial2.begin(115200,SERIAL_8N1, 18,19);
  ss.begin(GPSBaud);
  sensors.begin();
  //Define all button functions
  p0_Next.attachPush(p0_Next_Press, &p0_Next);
  p0_Prev.attachPush(p0_Prev_Press, &p0_Prev);
  p0_Play.attachPush(p0_Play_Press, &p0_Play);
  p0_Pause.attachPush(p0_Pause_Press, &p0_Pause);
  //Ultrasonic
  UltrasonicProj::setUpFunctionUltrasonics();
  //Inside Temp (onboard)
  mySensor.begin(0x60);
  //LED
  pinMode(ledPinR, OUTPUT);
  pinMode(ledPinL, OUTPUT);
  interrupts();
  xTaskCreate(
    getUltrasonics,      // Function that should be called
    "Get Ultrasonics",    // Name of the task (for debugging)
    4096,               // Stack size (bytes)
    NULL,               // Parameter to pass
    3,                  // Task priority
    &taskHandle1               // Task handle
  );
  xTaskCreate(
    do_every_so_often1,      // Function that should be called
    "Sending Nextion data 1",    // Name of the task (for debugging)
    4096,               // Stack size (bytes)
    NULL,               // Parameter to pass
    3,                  // Task priority
    &taskHandle2               // Task handle
  );
  xTaskCreate(
    do_every_so_often2,      // Function that should be called
    "Sending Nextion data 2",    // Name of the task (for debugging)
    4096,               // Stack size (bytes)
    NULL,               // Parameter to pass
    3,                  // Task priority
    &taskHandle2               // Task handle
  );
  xTaskCreate(
    GPSAvailable,      // Function that should be called
    "GPS",    // Name of the task (for debugging)
    2048,               // Stack size (bytes)
    NULL,               // Parameter to pass
    2,                  // Task priority
    &taskHandle3               // Task handle
  );
  xTaskCreate(
    coolantTemp,      // Function that should be called
    "Coolant Data",    // Name of the task (for debugging)
    4096,               // Stack size (bytes)
    NULL,               // Parameter to pass
    1,                  // Task priority
    &taskHandle4               // Task handle
  );
  xTaskCreate(
    InsideTemp,      // Function that should be called
    "Inside Temp",    // Name of the task (for debugging)
    2048,               // Stack size (bytes)
    NULL,               // Parameter to pass
    1,                  // Task priority
    &taskHandle5               // Task handle
  );
  xTaskCreate(
    NextionList,      // Function that should be called
    "Nextion touch events",    // Name of the task (for debugging)
    2048,               // Stack size (bytes)
    NULL,               // Parameter to pass
    4,                  // Task priority
    &taskHandle6               // Task handle
  );
}


void loop() {
  //Do nothing because RTOS handles
}






